==> kDiff-2.0.0-sources.jar: pw/prok/kdiff/delta/Delta.java <==
/*
   Copyright 2010 Dmitry Naumenko (dm.naumenko@gmail.com)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package pw.prok.kdiff.delta;

import pw.prok.kdiff.Chunk;
import pw.prok.kdiff.PatchFailedException;

import java.util.List;

/**
 * Describes the delta between original and revised texts.
 *
 * @param <T> The type of the compared elements in the 'lines'.
 */
public abstract class Delta<T> {
    /**
     * The original chunk.
     */
    private Chunk<T> original;

    /**
     * The revised chunk.
     */
    private Chunk<T> revised;

    /**
     * Specifies the type of the delta.
     */
    public enum DeltaType {
        /**
         * A change in the original.
         */
        CHANGE,
        /**
         * A delete from the original.
         */
        DELETE,
        /**
         * An insert into the original.
         */
        INSERT
    }

    /**
     * Construct the delta for original and revised chunks
     *
     * @param original Chunk describing the original text. Must not be {@code null}.
     * @param revised  Chunk describing the revised text. Must not be {@code null}.
     */
    public Delta(Chunk<T> original, Chunk<T> revised) {
        if (original == null) {
            throw new IllegalArgumentException("original must not be null");
        }
        if (revised == null) {
            throw new IllegalArgumentException("revised must not be null");
        }
        this.original = original;
        this.revised = revised;
    }

    /**
     * Verifies that this delta can be used to patch the given text.
     *
     * @param target the text to patch.
     * @throws PatchFailedException if the patch cannot be applied.
     */
    public abstract void verify(List<T> target) throws PatchFailedException;

    /**
     * Applies this delta as the patch for a given target
     *
     * @param target the given target
     * @throws PatchFailedException if the patch cannot be applied.
     */
    public abstract void applyTo(List<T> target) throws PatchFailedException;

    /**
     * Cancel this delta for a given revised text. The action is opposite to
     * patch.
     *
     * @param target the given revised text
     */
    public abstract void restore(List<T> target);

    /**
     * Returns the type of delta
     *
     * @return the type enum
     */
    public abstract DeltaType getType();

    /**
     * @return The Chunk describing the original text.
     */
    public Chunk<T> getOriginal() {
        return original;
    }

    /**
     * @param original The Chunk describing the original text to set.
     */
    public void setOriginal(Chunk<T> original) {
        this.original = original;
    }

    /**
     * @return The Chunk describing the revised text.
     */
    public Chunk<T> getRevised() {
        return revised;
    }

    /**
     * @param revised The Chunk describing the revised text to set.
     */
    public void setRevised(Chunk<T> revised) {
        this.revised = revised;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((original == null) ? 0 : original.hashCode());
        result = prime * result + ((revised == null) ? 0 : revised.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        Delta<T> other = (Delta) obj;
        if (original == null) {
            if (other.original != null)
                return false;
        } else if (!original.equals(other.original))
            return false;
        if (revised == null) {
            if (other.revised != null)
                return false;
        } else if (!revised.equals(other.revised))
            return false;
        return true;
    }

    public abstract Delta<T> copy();
}

==> kDiff-2.1.0-sources.jar: pw/prok/kdiff/delta/Delta.java <==
/*
   Copyright 2010 Dmitry Naumenko (dm.naumenko@gmail.com)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package pw.prok.kdiff.delta;

import pw.prok.kdiff.Chunk;
import pw.prok.kdiff.Hunk;
import pw.prok.kdiff.PatchFailedException;
import pw.prok.kdiff.diff.DiffException;

import java.util.List;

/**
 * Describes the delta between original and revised texts.
 *
 * @param <T> The type of the compared elements in the 'lines'.
 */
public class Delta<T> {
    /**
     * The original chunk.
     */
    private Chunk<T> original;

    /**
     * The revised chunk.
     */
    private Chunk<T> revised;

    /**
     * Specifies the type of the delta.
     */
    public enum DeltaType {
        /**
         * A change in the original.
         */
        CHANGE,
        /**
         * A delete from the original.
         */
        DELETE,
        /**
         * An insert into the original.
         */
        INSERT
    }

    /**
     * Construct the delta for original and revised chunks
     *
     * @param original Chunk describing the original text. Must not be {@code null}.
     * @param revised  Chunk describing the revised text. Must not be {@code null}.
     */
    public Delta(Chunk<T> original, Chunk<T> revised) {
        if (original == null) {
            throw new IllegalArgumentException("original must not be null");
        }
        if (revised == null) {
            throw new IllegalArgumentException("revised must not be null");
        }
        this.original = original;
        this.revised = revised;
    }

    /**
     * Applies this delta as the patch for a given target
     *
     * @param target the given target
     * @return hunk instance of are required, null otherwise
     * @throws PatchFailedException if the patch cannot be applied.
     */
    public Hunk<T> patch(List<T> target) throws PatchFailedException {
        try {
            final int offset = getOriginal().applyHunk(target);
            final int original = getOriginal().getPosition();
            patch(target, original + offset, getOriginal().getLines(), getRevised().getLines());
            return offset == 0 ? null : new Hunk<>(this, offset);
        } catch (DiffException e) {
            throw new PatchFailedException(e);
        }
    }

    /**
     * Cancel this delta for a given revised text. The action is opposite to
     * patch.
     *
     * @param target the given revised text
     * @return hunk instance of are required, null otherwise
     * @throws PatchFailedException if cannot find elements to patch
     */
    public Hunk<T> restore(List<T> target) throws PatchFailedException {
        try {
            final int offset = getRevised().applyHunk(target);
            final int revised = getRevised().getPosition();
            patch(target, revised + offset, getRevised().getLines(), getOriginal().getLines());
            return offset == 0 ? null : new Hunk<>(this, offset);
        } catch (DiffException e) {
            throw new PatchFailedException(e);
        }
    }

    private static <T> void patch(List<T> target, int position, List<T> from, List<T> to) {
        target.subList(position, position + from.size()).clear();
        target.addAll(position, to);
    }

    /**
     * Returns the type of delta
     *
     * @return the type enum
     */
    public DeltaType getType() {
        if (getOriginal().size() == 0) return DeltaType.INSERT;
        if (getRevised().size() == 0) return DeltaType.DELETE;
        return DeltaType.CHANGE;
    }

    /**
     * @return The Chunk describing the original text.
     */
    public Chunk<T> getOriginal() {
        return original;
    }

    /**
     * @param original The Chunk describing the original text to set.
     */
    public void setOriginal(Chunk<T> original) {
        this.original = original;
    }

    /**
     * @return The Chunk describing the revised text.
     */
    public Chunk<T> getRevised() {
        return revised;
    }

    /**
     * @param revised The Chunk describing the revised text to set.
     */
    public void setRevised(Chunk<T> revised) {
        this.revised = revised;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((original == null) ? 0 : original.hashCode());
        result = prime * result + ((revised == null) ? 0 : revised.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        Delta<T> other = (Delta) obj;
        if (original == null) {
            if (other.original != null)
                return false;
        } else if (!original.equals(other.original))
            return false;
        if (revised == null) {
            if (other.revised != null)
                return false;
        } else if (!revised.equals(other.revised))
            return false;
        return true;
    }

    public Delta<T> copy() {
        return new Delta<>(original.copy(), revised.copy());
    }
}
